@@ -18,23 +18,25 @@
  *                                                                         *
  ***************************************************************************/
 
+#include <errno.h>
 #include <string.h>
 #include <unistd.h>
 
 #ifdef linux
-  #include <linux/lp.h>
-  #include <sys/io.h>
+#include <sys/io.h>
 #else
-  #ifdef __CYGWIN__
-    #include <sys/io.h>
-  #else
-    #if __FreeBSD__
-      #include <dev/ppbus/ppi.h>
-      #include <dev/ppbus/ppbconf.h>
+#ifdef __CYGWIN__
+#include <sys/io.h>
+#else
+#if __FreeBSD__
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <dev/ppbus/ppi.h>
+#include <dev/ppbus/ppbconf.h>
 /* #else */
 /* #error This driver is not usable on your operation system. Sorry. */
-    #endif
-  #endif
+#endif
+#endif
 #endif
 
 #if defined(linux) || defined(__CYGWIN__) || defined(__FreeBSD__)
@@ -66,17 +68,17 @@
 /***************************************************************/
 
 /* signals on the S88-Bus */
-#define S88_QUIET 0x00 /* all signals low */
-#define S88_RESET 0x04 /* reset signal high */
-#define S88_LOAD  0x02 /* load signal high */
-#define S88_CLOCK 0x01 /* clock signal high */
-#define S88_DATA1 0x40 /* mask for data form S88 bus 1 (ACK) */
-#define S88_DATA2 0x80 /* mask for data from S88 bus 2 (BUSY) !inverted */
-#define S88_DATA3 0x20 /* mask for data from S88 bus 3 (PEND) */
-#define S88_DATA4 0x10 /* mask for data from S88 bus 4 (SEL) */
+#define S88_QUIET 0x00          /* all signals low */
+#define S88_RESET 0x04          /* reset signal high */
+#define S88_LOAD  0x02          /* load signal high */
+#define S88_CLOCK 0x01          /* clock signal high */
+#define S88_DATA1 0x40          /* mask for data form S88 bus 1 (ACK) */
+#define S88_DATA2 0x80          /* mask for data from S88 bus 2 (BUSY) !inverted */
+#define S88_DATA3 0x20          /* mask for data from S88 bus 3 (PEND) */
+#define S88_DATA4 0x10          /* mask for data from S88 bus 4 (SEL) */
 
 /* Output a Signal to the Bus */
-#define S88_WRITE(x) for (i = 0; i < S88CLOCK_SCALE; i++) outb(x, S88PORT)
+#define S88_WRITE(x) for (int i = 0; i < S88CLOCK_SCALE; i++) outb(x, S88PORT)
 
 /* possible io-addresses for the parallel port */
 static const unsigned long int LPT_BASE[] = { 0x378, 0x278, 0x3BC };
@@ -88,6 +90,7 @@ static const unsigned char BIT_VALUES[] =
     { 0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01 };
 
 #define __ddl_s88 ((DDL_S88_DATA *) buses[busnumber].driverdata)
+#define __ddl_s88t ((DDL_S88_DATA *) buses[btd->bus].driverdata)
 
 int readconfig_DDL_S88(xmlDocPtr doc, xmlNodePtr node, bus_t busnumber)
 {
@@ -97,7 +100,7 @@ int readconfig_DDL_S88(xmlDocPtr doc, xmlNodePtr node, bus_t busnumber)
 
     if (buses[busnumber].driverdata == NULL) {
         syslog_bus(busnumber, DBG_ERROR,
-                "Memory allocation error in module '%s'.", node->name);
+                   "Memory allocation error in module '%s'.", node->name);
         return 0;
     }
 
@@ -128,7 +131,9 @@ int readconfig_DDL_S88(xmlDocPtr doc, xmlNodePtr node, bus_t busnumber)
     __ddl_s88->number_fb[3] = 1;
 
     for (i = 1; i < 4; i++) {
+        strcpy(buses[busnumber + i].description, "FB");
         buses[busnumber + i].type = SERVER_S88;
+        buses[busnumber + i].debuglevel = buses[busnumber].debuglevel;
         buses[busnumber + i].init_func = NULL;
         buses[busnumber + i].thr_func = &thr_sendrec_dummy;
         buses[busnumber + i].driverdata = NULL;
@@ -138,8 +143,10 @@ int readconfig_DDL_S88(xmlDocPtr doc, xmlNodePtr node, bus_t busnumber)
     xmlChar *txt = NULL;
 
     while (child != NULL) {
-        if (xmlStrncmp(child->name, BAD_CAST "text", 4) == 0) {
-            /* just do nothing, it is only a comment */
+
+        if ((xmlStrncmp(child->name, BAD_CAST "text", 4) == 0) ||
+            (xmlStrncmp(child->name, BAD_CAST "comment", 7) == 0)) {
+            /* just do nothing, it is only formatting text or a comment */
         }
 
         else if (xmlStrcmp(child->name, BAD_CAST "ioport") == 0) {
@@ -209,28 +216,26 @@ int readconfig_DDL_S88(xmlDocPtr doc, xmlNodePtr node, bus_t busnumber)
 
         else
             syslog_bus(busnumber, DBG_WARN,
-                    "WARNING, unknown tag found: \"%s\"!\n",
-                    child->name);;
+                       "WARNING, unknown tag found: \"%s\"!\n",
+                       child->name);;
 
         child = child->next;
     }
 
-    if (init_FB(busnumber, __ddl_s88->number_fb[0] * 16)) {
-        __ddl_s88->number_fb[0] = 0;
-        syslog_bus(busnumber, DBG_ERROR, "Can't create array for feedback");
-    }
-    if (init_FB(busnumber, __ddl_s88->number_fb[1] * 16)) {
-        __ddl_s88->number_fb[1] = 0;
-        syslog_bus(busnumber, DBG_ERROR, "Can't create array for feedback");
-    }
-    if (init_FB(busnumber, __ddl_s88->number_fb[2] * 16)) {
-        __ddl_s88->number_fb[2] = 0;
-        syslog_bus(busnumber, DBG_ERROR, "Can't create array for feedback");
-    }
-    if (init_FB(busnumber, __ddl_s88->number_fb[3] * 16)) {
-        __ddl_s88->number_fb[3] = 0;
-        syslog_bus(busnumber, DBG_ERROR, "Can't create array for feedback");
+    for (i = 0; i < 4; i++) {
+        if (init_FB(busnumber + i, __ddl_s88->number_fb[i] * 16)) {
+            __ddl_s88->number_fb[i] = 0;
+            syslog_bus(busnumber + i, DBG_ERROR,
+                       "Can't create array for s88-feedback "
+                       "channel %d", i + 1);
+        }
+        else
+            syslog_bus(busnumber + i, DBG_INFO,
+                       "%d feeback contacts for channel %d successfully "
+                       "initialized.", __ddl_s88->number_fb[i] * 16,
+                       i + 1);
     }
+
     return (4);
 }
 
@@ -252,10 +257,11 @@ int readconfig_DDL_S88(xmlDocPtr doc, xmlNodePtr node, bus_t busnumber)
 ****************************************************************/
 int init_bus_S88(bus_t busnumber)
 {
-    unsigned int i;             /* loop counter */
+    int result;
     int isin = 0;               /* reminder for checking */
     int S88PORT = __ddl_s88->port;
     int S88CLOCK_SCALE = __ddl_s88->clockscale;
+
 #ifdef linux
     syslog_bus(busnumber, DBG_INFO, "init_bus DDL(Linux) S88");
 #else
@@ -269,11 +275,18 @@ int init_bus_S88(bus_t busnumber)
         return 1;
     }
     /* test, whether S88DEV is a valid io-address for a parallel device */
-    for (i = 0; i < LPT_NUM; i++)
+    for (unsigned int i = 0; i < LPT_NUM; i++)
         isin = isin || (S88PORT == LPT_BASE[i]);
     if (isin) {
-        /* test if port is accessible*/
-        if (ioperm(S88PORT, 3, 1) == 0) {
+        /* test if port is accessible */
+        result = ioperm(S88PORT, 3, 1);
+        if (result == -1) {
+            syslog_bus(busnumber, DBG_FATAL,
+                       "ioperm() failed: %s (errno = %d).",
+                       strerror(result), result);
+            return 1;
+        }
+        else {
             /* test, whether there is a real device on the S88DEV-port
                by writing and reading data to the port. If the written
                data is returned, a real port is there
@@ -291,26 +304,22 @@ int init_bus_S88(bus_t busnumber)
             }
             else {
                 syslog_bus(busnumber, DBG_WARN,
-                    "warning: There is no port for s88 at 0x%X.", S88PORT);
+                           "Warning: There is no port for s88 at 0x%X.",
+                           S88PORT);
                 /* stop access to port address */
                 ioperm(S88PORT, 3, 0);
                 return 1;
             }
         }
-        else {
-            syslog_bus(busnumber, DBG_ERROR,
-                "warning: Access to port 0x%X denied.", S88PORT);
-            return 1;
-        }
     }
     else {
         syslog_bus(busnumber, DBG_WARN,
-            "warning: 0x%X is not valid port address for s88 device.",
-            S88PORT);
+                   "Warning: 0x%X is not valid port address for s88 device.",
+                   S88PORT);
         return 1;
     }
-    syslog_bus(busnumber, DBG_INFO, "s88 port successfully initialized at 0x%X.",
-        S88PORT);
+    syslog_bus(busnumber, DBG_INFO,
+               "s88 port successfully initialized at 0x%X.", S88PORT);
     return 0;
 }
 
@@ -330,9 +339,9 @@ int init_bus_S88(bus_t busnumber)
 ****************************************************************/
 void s88load(bus_t busnumber)
 {
-    int i, j, k, inbyte;
+    int j, k, inbyte;
     struct timeval nowtime;
-    unsigned int s88data[S88_MAXPORTSB * S88_MAXBUSSES]; /* valid bus-data */
+    unsigned int s88data[S88_MAXPORTSB * S88_MAXBUSSES];        /* valid bus-data */
     int S88PORT = __ddl_s88->port;
     int S88CLOCK_SCALE = __ddl_s88->clockscale;
     int S88REFRESH = 1000 * __ddl_s88->refresh;
@@ -395,29 +404,29 @@ void s88load(bus_t busnumber)
 }
 
 /*thread cleanup routine for this bus*/
-static void end_bus_thread(bus_thread_t *btd)
+static void end_bus_thread(bus_thread_t * btd)
 {
     int result;
 
     syslog_bus(btd->bus, DBG_INFO, "DDL-S88 bus terminated.");
 
 #ifdef __FreeBSD__
-    if (((DDL_S88_DATA *) buses[busnumber].driverdata)->Fd != -1)
-        close(((DDL_S88_DATA *) buses[busnumber].driverdata)->Fd);
+    if (((DDL_S88_DATA *) buses[btd->bus].driverdata)->Fd != -1)
+        close(((DDL_S88_DATA *) buses[btd->bus].driverdata)->Fd);
 #endif
 
     result = pthread_mutex_destroy(&buses[btd->bus].transmit_mutex);
     if (result != 0) {
         syslog_bus(btd->bus, DBG_WARN,
-                "pthread_mutex_destroy() failed: %s (errno = %d).",
-                strerror(result), result);
+                   "pthread_mutex_destroy() failed: %s (errno = %d).",
+                   strerror(result), result);
     }
 
     result = pthread_cond_destroy(&buses[btd->bus].transmit_cond);
     if (result != 0) {
         syslog_bus(btd->bus, DBG_WARN,
-                "pthread_mutex_init() failed: %s (errno = %d).",
-                strerror(result), result);
+                   "pthread_mutex_init() failed: %s (errno = %d).",
+                   strerror(result), result);
     }
 
     free(buses[btd->bus].driverdata);
@@ -428,29 +437,30 @@ void *thr_sendrec_S88(void *v)
 {
     int last_cancel_state, last_cancel_type;
 
-    bus_thread_t* btd = (bus_thread_t*) malloc(sizeof(bus_thread_t));
+    bus_thread_t *btd = (bus_thread_t *) malloc(sizeof(bus_thread_t));
     if (btd == NULL)
-        pthread_exit((void*) 1);
-    btd->bus =  (bus_t) v;
+        pthread_exit((void *) 1);
+    btd->bus = (bus_t) v;
     btd->fd = -1;
 
     pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, &last_cancel_state);
     pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, &last_cancel_type);
 
-    /*register cleanup routine*/
+    /*register cleanup routine */
     pthread_cleanup_push((void *) end_bus_thread, (void *) btd);
 
     unsigned long int sleepusec = 100000;
 
-    int S88REFRESH = ((DDL_S88_DATA *) buses[btd->bus].driverdata)->refresh;
+    unsigned long int S88REFRESH =
+        ((DDL_S88_DATA *) buses[btd->bus].driverdata)->refresh;
     /* set refresh-cycle */
     if (sleepusec < S88REFRESH * 1000)
         sleepusec = S88REFRESH * 1000;
 
-    syslog_bus(btd->bus, DBG_INFO, "DDL_S88 bus startet (device = %s).",
-        buses[btd->bus].device.file.path);
+    syslog_bus(btd->bus, DBG_INFO, "DDL_S88 bus startet (device = %04x).",
+               __ddl_s88t->port);
 
-    while (1) {
+    while (true) {
         if (buses[btd->bus].power_changed == 1) {
             char msg[110];
             buses[btd->bus].power_changed = 0;
@@ -458,26 +468,41 @@ void *thr_sendrec_S88(void *v)
             enqueueInfoMessage(msg);
         }
 
-        /*do nothing if power is off*/
+        /*do nothing if power is off */
         if (buses[btd->bus].power_state == 0) {
-            usleep(1000);
+            if (usleep(1000) == -1) {
+                syslog_bus(btd->bus, DBG_ERROR,
+                           "usleep() failed: %s (errno = %d)\n",
+                           strerror(errno), errno);
+            }
             continue;
         }
 
         check_reset_fb(btd->bus);
-        usleep(sleepusec);
+        if (usleep(sleepusec) == -1) {
+            syslog_bus(btd->bus, DBG_ERROR,
+                       "usleep() failed: %s (errno = %d)\n",
+                       strerror(errno), errno);
+        }
         s88load(btd->bus);
     }
 
-    /*run the cleanup routine*/
+    /*run the cleanup routine */
     pthread_cleanup_pop(1);
     return NULL;
 }
 
 void *thr_sendrec_dummy(void *v)
 {
-    while (1)
-        sleep(1);
+    int result;
+
+    while (true) {
+        result = sleep(1);
+        if (result != 0) {
+            syslog_bus(0, DBG_ERROR,
+                       "sleep() interrupted, %d seconds left\n", result);
+        }
+    }
 }
 
 /*---------------------------------------------------------------------------
@@ -502,12 +527,12 @@ int FBSD_ioperm(int Port, int KeineAhnung, int DesiredAccess,
     if (DesiredAccess == 0) {
         if (__ddl_s88->Fd != -1) {
             close(__ddl_s88->Fd);
-            syslog_bus(busnumber, DBG_DEBUG, "FBSD DDL-S88 closing port %04X",
-                Port);
+            syslog_bus(busnumber, DBG_DEBUG,
+                       "FBSD DDL-S88 closing port %04X", Port);
         }
         else {
             syslog_bus(busnumber, DBG_WARN,
-                "FBSD DDL-S88 closing NOT OPEN port %04X", Port);
+                       "FBSD DDL-S88 closing NOT OPEN port %04X", Port);
         }
         __ddl_s88->Fd = -1;
         return 0;
@@ -537,18 +562,20 @@ int FBSD_ioperm(int Port, int KeineAhnung, int DesiredAccess,
     if (found == 0)
         return -1;
 
-    sprintf(DevName, "/dev/ppi%d", i);
+    snprintf(DevName, sizeof(DevName), "/dev/ppi%d", i);
 
     Fd = open(DevName, O_RDWR);
 
     if (Fd < 0) {
-        syslog_bus(busnumber, DBG_ERROR, "FBSD DDL-S88 open port %04X on %s "
-                "failed: %s (errno = %d).\n", Port, DevName,
-                strerror(errno), errno);
+        syslog_bus(busnumber, DBG_ERROR,
+                   "FBSD DDL-S88 open port %04X on %s "
+                   "failed: %s (errno = %d).\n", Port, DevName,
+                   strerror(errno), errno);
         return Fd;
     }
     syslog_bus(busnumber, DBG_INFO,
-        "FBSD DDL-S88 success opening port %04X on %s", Port, DevName);
+               "FBSD DDL-S88 success opening port %04X on %s", Port,
+               DevName);
 
     __ddl_s88->Fd = Fd;
     return 0;
@@ -567,8 +594,8 @@ unsigned char FBSD_inb(int Woher, bus_t busnumber)
     /* syslog_bus(busnumber, DBG_DEBUG, "FBSD DDL-S88 InB start on port %04X",Woher); */
     if (__ddl_s88->Fd == -1) {
         syslog_bus(busnumber, DBG_ERROR,
-            "FBSD DDL-S88 Device not open for reading");
-        exit(1);
+                   "FBSD DDL-S88 Device not open for reading");
+        exit(EXIT_FAILURE);
     }
 
     WelchesPort = Woher - __ddl_s88->port;
@@ -585,14 +612,14 @@ unsigned char FBSD_inb(int Woher, bus_t busnumber)
             break;
         default:
             syslog_bus(busnumber, DBG_FATAL, "FBSD DDL-S88 Read access to "
-                    "port %04X requested, not applicable!", Woher);
+                       "port %04X requested, not applicable!", Woher);
             return 0;
     }
     result = ioctl(__ddl_s88->Fd, WelcherIoctl, &i);
     if (result == -1) {
         syslog_bus(busnumber, DBG_ERROR,
-                "ioctl() failed: %s (errno = %d)\n",
-                strerror(errno), errno);
+                   "ioctl() failed: %s (errno = %d)\n",
+                   strerror(errno), errno);
     }
     /* syslog_bus(busnumber, DBG_DEBUG, "FBSD DDL-S88 InB finished Data %02X",i); */
     return i;
@@ -608,8 +635,9 @@ unsigned char FBSD_outb(unsigned char Data, int Wohin, bus_t busnumber)
     /* syslog_bus(busnumber, DBG_DEBUG, "FBSD DDL-S88 OutB %d on Port %04X",Data,Wohin); */
     if (__ddl_s88->Fd == -1) {
         syslog_bus(busnumber, DBG_ERROR,
-            "FBSD DDL-S88 Device not open for writing byte %d", Data);
-        exit(1);
+                   "FBSD DDL-S88 Device not open for writing byte %d",
+                   Data);
+        exit(EXIT_FAILURE);
         return -1;
     }
 
@@ -626,15 +654,16 @@ unsigned char FBSD_outb(unsigned char Data, int Wohin, bus_t busnumber)
             WelcherIoctl = PPISCTRL;
             break;
         default:
-            syslog_bus(busnumber, DBG_FATAL, "FBSD DDL-S88 Write access to "
-                    "port %04X requested, not applicable!", Wohin);
+            syslog_bus(busnumber, DBG_FATAL,
+                       "FBSD DDL-S88 Write access to "
+                       "port %04X requested, not applicable!", Wohin);
             return 0;
     }
     result = ioctl(__ddl_s88->Fd, WelcherIoctl, &Data);
     if (result == -1) {
         syslog_bus(busnumber, DBG_ERROR,
-                "ioctl() failed: %s (errno = %d)\n",
-                strerror(errno), errno);
+                   "ioctl() failed: %s (errno = %d)\n",
+                   strerror(errno), errno);
     }
     /* syslog_bus(busnumber, DBG_DEBUG, "FBSD DDL-S88 OutB finished"); */
     return Data;
